;; bship.scm
;; Debajit Ghosh
;; 1/30/96

(define my-view (new-SoXtExaminerViewer))  ; viewer for your own board
                                           ; in this version, everything
                                           ; happens here
(define opp-view (new-SoXtExaminerViewer)) ; not used yet

(define root (new-SoSelection))

(-> my-view 'setSceneGraph root)
(-> my-view 'setTitle "Your Board")
(-> my-view 'setViewing 0)
(-> my-view 'show)


;; generate coordinates for an dim x dim board

(define (gen-coords x y dim)
  (if (< x dim)
      (if (< y dim)
	  (cons
	   (list (+ 0.5 x) (+ 0.5 y) 0 (string-append "_"
						      (number->string
						       (+ (* x dim) y))))
	   (gen-coords x (+ y 1) dim))
	  (gen-coords (+ x 1) 0 dim))
      '()))

;;; SICP-ish simple object system for dealing similarly
     ;;; with Scheme and C++ objects using "send"

(define (send object message . args)
  (if (C++-object? object)
      (eval `(-> ,object ',message ,@args))
      (let ((method (get-method object message)))
	(if (not (no-method? method))
	    (apply method (cons object args))
	    (error "No method named" message)))))

(define (get-method object message)
  (object message))

(define (no-method method)
  'no-method)

(define (no-method? method)
  (eq? method 'no-method))


;; constructor for the board object

(define (new-Board dim)
  (define root (new-SoSeparator))
  (send root 'ref)
  (define coordinates (gen-coords 0 0 dim))

  (lambda (message)
         ;;; Method to return pointer to root of scene graph
    (cond ((eq? message 'getGeometry)
	   (lambda (self)
	     root))
	  ;;; get the coordinates of nodes on this board
	  ((eq? message 'getCoords)
	   (lambda (self)
	     coordinates))
	  ;;; generate the board
	  ((eq? message 'generate)
	   (lambda (self)
	     (let loop
		 ((coords coordinates))
	       (if (not (null? coords))
		   (let ((transvalue (car coords)))
		     (begin
		       (display transvalue)
		       (define s (new-SoSeparator))
		       (send root 'addChild s)
		       (define t (new-SoTransform))
		       (send s 'addChild t)
		       (send (send t 'translation) 'setValue (car transvalue)
			     (cadr transvalue) (caddr transvalue))
		       (define m (new-SoMaterial))
		       (send s 'addChild m)
		       (define node (new-SoSphere))
		       (send (send node 'radius) 'setValue 0.1)
		       (send node 'setname (new-sbname (cadddr transvalue)))
		       (send s 'addChild node)
		       (loop (cdr coords))))))))
	  ;;; turn a node that has been "hit" blue
	  ((eq? message 'hitNode)
	   (lambda (self index)
	     (let* ((sel (SoSeparator-cast (send root 'getChild index)))
		    (mat (SoMaterial-cast (send sel 'getChild 1))))
	       (send (send mat 'diffuseColor) 'setValue 0.0 0.0 1.0))))
	       
	  (else (no-method message)))))

;; set up everything

(define board (new-Board 3))
(send root 'addChild (send board 'getGeometry))
(send board 'generate)


;; set up selection callback
(define callback-info (new-SchemeSoCBInfo))
(-> callback-info 'ref)
(-> (-> callback-info 'callbackName) 'setValue "selection-callback")
(-> root 'addSelectionCallback (get-scheme-selection-path-cb) 
    (void-cast callback-info))

(define (selection-callback data path)
       (let* ((selected-object (-> path 'getTail))
	      (sel-obj-pop (-> path 'pop))
	      (sel-obj-par (SoSeparator-cast (-> path 'getTail))))
         (if (= 1 (-> selected-object 'isOfType (SoSphere::getClassTypeId)))
             (let* ((the-sphere (SoSphere-cast selected-object))
		    (sphere-name (-> the-sphere 'getName))
		    (sphere-string (-> sphere-name 'getString))
		    (sphere-mat (SoMaterial-cast (-> sel-obj-par 'getChild 1))))
	       (display sphere-string)
	       (-> (-> sphere-mat 'diffuseColor) 'setValue 1.0 0.0 0.0)
               (newline)
	       (send-over-network sphere-string)))))

;;; idle callback, basically check to see if any "hits" have been made
;;; during idle cycles

(define idle-cb
  (lambda (user-data sensor)
    ;; Tell sensor to reschedule
    (begin
      (send sensor 'schedule)
      (let ((index-string (get-from-network)))
	(if (not (null? index-string))
	    (send board 'hitNode (string->number 
				  (substring index-string
					     1
					     (string-length index-string)))))))))

;; Callback information node for the idle sensor's callback
(define cb-info (new-SchemeSoCBInfo))
(-> cb-info 'ref)
(-> (-> cb-info 'callbackName) 'setValue "idle-cb")

;; Creation and scheduling of the idle sensor
(define idle-sensor (new-SoIdleSensor (get-scheme-sensor-cb) 
				      (void-cast cb-info)))
(-> idle-sensor 'schedule)
